import datetime
import sys
import time
from pathlib import Path

import httpx
import typer
from rich import print
from rich.console import Console
from rich.panel import Panel

from taas_client import __version__

console = Console()
app = typer.Typer()

BASE_URL = "http://localhost:5000"


def version_callback(value: bool) -> None:
    if not value:
        return

    print(f"taas-client: {__version__}")

    # FIXME how could we use the given URL argument here too?
    r = httpx.get(f"{BASE_URL}/version")
    r.raise_for_status()
    server_version = r.json()["version"]
    print(f"taas server: {server_version}")

    raise typer.Exit()


def check_notebook_file(path: Path, force: bool) -> None:
    if not path.is_relative_to(Path.home() / "work" / "AutPy"):
        console.print(
            f"[red]Refusing to submit file from unexpected location[/]: {path}\n"
            "Make sure you're not accidentally working in the 'Originale' folder!"
        )
        if not force:
            raise typer.Exit(code=1)

    modified = round(time.time() - path.stat().st_mtime)
    sec = "second" if modified == 1 else "seconds"
    suffix = ", [red]make sure you saved![/]" if modified >= 10 else ""
    console.print(f"Last change: [bold]{modified}[/bold] {sec} ago{suffix}")


def check_project_file(path: Path, force: bool) -> None:
    pass


@app.command()
def upload_file(
    notebook: Path = typer.Argument(
        ...,
        exists=True,
        file_okay=True,
        dir_okay=False,
        writable=False,
        readable=True,
        resolve_path=True,
    ),
    url: str = typer.Argument(f"{BASE_URL}/testbook", envvar="TAAS_URL"),
    project: bool = False,
    force: bool = False,
    _: bool = typer.Option(False, "--version", callback=version_callback),
) -> None:
    if project:
        # dirty hack because url is fixed for testbook at the moment.
        # After the module it needs to be changed to a base url
        url = f"{url[:-8]}project"

    if project:
        check_project_file(path=notebook, force=force)
    else:
        check_notebook_file(path=notebook, force=force)

    files = {"file": open(notebook, "rb")}
    try:
        text = "Uploading" if project else "Testing"
        with console.status(f"[bold green]{text}..."):
            r = httpx.post(url, files=files, timeout=600.0)
        r.raise_for_status()

        result = r.json()
        test_result = result["tests"]
        for status, style in [
            ("error", "bold red"),
            ("failed", "red"),
            ("passed", "green"),
            ("skipped", ""),
            ("xfailed", "red"),
            ("xpassed", ""),
        ]:
            if test_result[status]:
                lines = [
                    f" -  test_{test['nodeid']} {test['outcome']} at {test['when']}"
                    if test["when"] != "call"
                    else f" -  test_{test['nodeid']} {test['outcome']}"
                    for test in test_result[status]
                ]
                console.print(
                    Panel(
                        "\n".join(lines),
                        title=status.capitalize(),
                        subtitle=f"{len(test_result[status])}",
                        style=style,
                    )
                )
    except httpx.HTTPError as exc:
        print(f"[red bold]{exc}[/]", file=sys.stderr)
        try:
            data = exc.response.json()  # type: ignore[attr-defined]
            print(f"[red]Error: {data.get('error')}[/]", file=sys.stderr)
            print(f"[red]Message: {data.get('msg')}[/]", file=sys.stderr)
            timestamp = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            print(f":boom: [bold]Submission failed![/] [white]({timestamp})[/]")
        except Exception:  # FIXME
            pass
        raise typer.Exit(code=1)

    commit = result.get("commit", "?")
    timestamp = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    print(
        ":mailbox: :heavy_check_mark: [bold]Submission successful![/] "
        f"[white]({timestamp}, {commit})[/]"
    )


if __name__ == "__main__":
    app()  # pragma: no cover
